<?php
require_once ('op5/sysinfo.php');
require_once ('op5/mayi.php');

class op5license_Constraints implements op5MayI_Constraints {
	protected $license_backend = false;

	public function __construct() {
		$this->license_backend = new op5license();
	}
	/**
	 * Execute a action
	 *
	 * @param string $action
	 *        	name of the action, as "path.to.resource:action"
	 * @param array $env
	 *        	environment variables for the constraints
	 * @param array $messages
	 *        	referenced array to add messages to
	 * @param op5MayIMetric[] $metrics
	 *        	referenced array to add performance data to
	 */
	public function run($action, $env, &$messages, &$metrics) {
		list($resource, $method) = array_map(function($str) { return explode('.', $str); }, explode(':', $action) );

		if($resource[0] == 'ninja') {
			if(isset($resource[1]) && $resource[1] == 'session') {
				// We should handle this case within backbox, not within license, but it needs therefore to be handled from the license
			} else {
				return true; /* ninja resource namespace is meta-methods in ninja, which shouldn't be controlled by license */
			}
		}

		$raw_metrics = array();
		$result = $this->license_backend->run($action, $env, $messages, $raw_metrics);
		foreach($raw_metrics as $name => $values) {
			list($value, $min, $max) = $values;
			$metrics[$name] = new op5MayIMetric($value, $min, $max);
		}
		return $result;
	}
}

class ninja_license {
	/**
	 * Create hooks and initialize libraries
	 */
	public function __construct() {
		Event::add('ninja.get_branding', array ($this,'get_branding'));
		Event::add('ninja.show_login', array ($this,'show_login'));
		Event::add('system.pre_controller', array ($this,'cleanup_session'));

		$mayi = op5MayI::instance();
		$mayi->act_upon(new op5license_Constraints());
	}

	/**
	 * Hook to retrieve licensed branding
	 */
	public function get_branding () {

		$license = new op5license();
		$metadata = $license->get_metadata();
		$data = &Event::$data['branding'];

		if (!empty($metadata) && isset($metadata['branding'])) {

			$data['label'] = $metadata['branding'];
			$format = 'brands/%s.png';

			try {
				$data['image'] = ninja::add_path(sprintf($format, $data['label']), false, false);
				$data['label'] = '';
			} catch (FileLookupErrorException $e) {
				// If there is no image, use default image already set
				// and the message as a label for that image.
				$data['image'] = ninja::add_path('brands/op5.png', false, true);
			}

		}

	}


	/**
	 * Test if we are allowed to see ninja at all, or see the "blaffa"
	 */
	private function goto_blaffa() {
		return !op5mayi::instance()->run('ninja.session:create.blaffa');
	}

	/**
	 * Hook to replace login screen if system is unlicensed
	 */
	public function show_login() {
		$controller = Event::$data;
		if ($this->goto_blaffa()) {
			$controller->template = $controller->add_view(
				'license_call_me_maybe');
		}
	}
	/**
	 * Destroy session if unlicensed.
	 * That means, we shouldn't be able to be logged in at all to ninja
	 */
	public function cleanup_session() {
		if ($this->goto_blaffa()) {
			Session::destroy();

			Auth::disable_auth_subsystem();
		}
	}
}

new ninja_license();
